@@ -15,11 +15,11 @@ import (
 	"github.com/dchest/safefile"
 	"github.com/pborman/getopt/v2"
 
+	"sigsum.org/sigsum-go/internal/ui"
 	"sigsum.org/sigsum-go/internal/version"
 	"sigsum.org/sigsum-go/pkg/crypto"
 	"sigsum.org/sigsum-go/pkg/key"
 	"sigsum.org/sigsum-go/pkg/log"
-	"sigsum.org/sigsum-go/pkg/policy"
 	"sigsum.org/sigsum-go/pkg/proof"
 	"sigsum.org/sigsum-go/pkg/requests"
 	"sigsum.org/sigsum-go/pkg/submit"
@@ -31,6 +31,7 @@ type Settings struct {
 	rawHash      bool
 	keyFile      string
 	policyFile   string
+	policyName   string
 	leafHash     bool
 	diagnostics  string
 	inputFiles   []string
@@ -41,11 +42,89 @@ type Settings struct {
 	timeout      time.Duration
 }
 
-// Empty name for stdin
+// A LeafSink represents the action to take for input leaf requests,
+// either writing them to file, or submitting them to the log.
+// The name argument corresponds to the filename, or an empty string for stdin.
 type LeafSink func(name string, leaf *requests.Leaf)
+
+// The name argument corresponds to the filename, or an empty string for stdin.
 type LeafSkip func(name string, msg *crypto.Hash, publicKey *crypto.PublicKey) bool
+
+// A LeafSource represents the input processing, either reading signed
+// leaf requests from file, or signing leaf requests based on input files
+// to be logged. For each input, the item is first passed to the skip
+// callback, and if skip returns false, it is passed on to the sink
+// callback.
 type LeafSource func(skip LeafSkip, sink LeafSink)
 
+// This function prepares a source function that uses the given key to sign each message.
+func getLeafRequestSignerSource(keyFile string, inputFiles []string, rawHash bool) (LeafSource, string, error) {
+	signer, policyNameFromPubKey, err := key.ReadPrivateKeyFileWithPolicy(keyFile)
+	if err != nil {
+		log.Fatal("Reading key file failed: %v", err)
+	}
+	publicKey := signer.Public()
+	if len(inputFiles) == 0 {
+		// No input files, so we use stdin
+		return func(_ LeafSkip, sink LeafSink) {
+			msg, err := readMessage(os.Stdin, rawHash)
+			if err != nil {
+				log.Fatal("Reading message from stdin failed: %v", err)
+			}
+			signature, err := types.SignLeafMessage(signer, msg[:])
+			if err != nil {
+				log.Fatal("Signing failed: %v", err)
+			}
+			sink("", &requests.Leaf{Message: msg, Signature: signature, PublicKey: publicKey})
+		}, policyNameFromPubKey, nil
+	}
+	// There are input files, so we use them
+	return func(skip LeafSkip, sink LeafSink) {
+		for _, inputFile := range inputFiles {
+			msg := readMessageFile(inputFile, rawHash)
+			if skip(inputFile, &msg, &publicKey) {
+				continue
+			}
+			signature, err := types.SignLeafMessage(signer, msg[:])
+			if err != nil {
+				log.Fatal("Signing failed: %v", err)
+			}
+			sink(inputFile, &requests.Leaf{Message: msg, Signature: signature, PublicKey: publicKey})
+		}
+	}, policyNameFromPubKey, nil
+}
+
+// This function prepares a source function that will process leaf requests that already contain signatures.
+func getLeafRequestSource(inputFiles []string) (LeafSource, string, error) {
+	if len(inputFiles) == 0 {
+		// No input files, so we use stdin
+		return func(_ LeafSkip, sink LeafSink) {
+			leaf, err := readLeafRequest(os.Stdin)
+			if err != nil {
+				log.Fatal("Leaf request on stdin not valid: %v", err)
+			}
+			sink("", &leaf)
+		}, "", nil
+	}
+	// There are input files, so we use them
+	return func(skip LeafSkip, sink LeafSink) {
+		for _, inputFile := range inputFiles {
+			leaf, err := readLeafRequestFile(inputFile)
+			if err != nil {
+				log.Fatal("Leaf request %q not valid: %v", inputFile, err)
+			}
+			// Strip suffix.
+			inputFile = strings.TrimSuffix(inputFile, ".req")
+			if len(inputFile) == 0 {
+				log.Fatal("Invalid input file name %q", ".req")
+			}
+			if !skip(inputFile, &leaf.Message, &leaf.PublicKey) {
+				sink(inputFile, &leaf)
+			}
+		}
+	}, "", nil
+}
+
 func main() {
 	var settings Settings
 	settings.parse(os.Args)
@@ -54,73 +133,26 @@ func main() {
 	}
 
 	var source LeafSource
+	var policyNameFromPubKey string
+	var err error
 	if len(settings.keyFile) > 0 {
-		signer, err := key.ReadPrivateKeyFile(settings.keyFile)
-		if err != nil {
-			log.Fatal("reading key file failed: %v", err)
-		}
-		publicKey := signer.Public()
-		if len(settings.inputFiles) == 0 {
-			source = func(_ LeafSkip, sink LeafSink) {
-				msg, err := readMessage(os.Stdin, settings.rawHash)
-				if err != nil {
-					log.Fatal("Reading message from stdin failed: %v", err)
-				}
-				signature, err := types.SignLeafMessage(signer, msg[:])
-				if err != nil {
-					log.Fatal("Signing failed: %v", err)
-				}
-				sink("", &requests.Leaf{Message: msg, Signature: signature, PublicKey: publicKey})
-			}
-		} else {
-			source = func(skip LeafSkip, sink LeafSink) {
-				for _, inputFile := range settings.inputFiles {
-					msg := readMessageFile(inputFile, settings.rawHash)
-					if skip(inputFile, &msg, &publicKey) {
-						continue
-					}
-					signature, err := types.SignLeafMessage(signer, msg[:])
-					if err != nil {
-						log.Fatal("signing failed: %v", err)
-					}
-					sink(inputFile, &requests.Leaf{Message: msg, Signature: signature, PublicKey: publicKey})
-				}
-			}
-		}
+		source, policyNameFromPubKey, err = getLeafRequestSignerSource(settings.keyFile, settings.inputFiles, settings.rawHash)
 	} else {
-		if len(settings.inputFiles) == 0 {
-			source = func(_ LeafSkip, sink LeafSink) {
-				leaf, err := readLeafRequest(os.Stdin)
-				if err != nil {
-					log.Fatal("Leaf request on stdin not valid: %v", err)
-				}
-				sink("", &leaf)
-			}
-		} else {
-			source = func(skip LeafSkip, sink LeafSink) {
-				for _, inputFile := range settings.inputFiles {
-					leaf, err := readLeafRequestFile(inputFile)
-					if err != nil {
-						log.Fatal("Leaf request %q not valid: %v", inputFile, err)
-					}
-					// Strip suffix.
-					inputFile = strings.TrimSuffix(inputFile, ".req")
-					if len(inputFile) == 0 {
-						log.Fatal("Invalid input file name %q", ".req")
-					}
-					if !skip(inputFile, &leaf.Message, &leaf.PublicKey) {
-						sink(inputFile, &leaf)
-					}
-				}
-			}
-		}
+		source, policyNameFromPubKey, err = getLeafRequestSource(settings.inputFiles)
+	}
+	if err != nil {
+		log.Fatal("Error: %v", err)
 	}
 
-	if len(settings.policyFile) > 0 {
-		policy, err := policy.ReadPolicyFile(settings.policyFile)
-		if err != nil {
-			log.Fatal("Invalid policy file: %v", err)
-		}
+	policy, err := ui.SelectPolicy(ui.PolicyParams{
+		File:           settings.policyFile,
+		Name:           settings.policyName,
+		NameFromPubKey: policyNameFromPubKey,
+	})
+	if err != nil {
+		log.Fatal("Failed to select policy: %v", err)
+	}
+	if policy != nil {
 		config := submit.Config{Policy: policy,
 			Domain:  settings.tokenDomain,
 			Timeout: settings.timeout,
@@ -131,7 +163,7 @@ func main() {
 			var err error
 			config.RateLimitSigner, err = key.ReadPrivateKeyFile(settings.tokenKeyFile)
 			if err != nil {
-				log.Fatal("reading token key file failed: %v", err)
+				log.Fatal("Reading token key file failed: %v", err)
 			}
 			// Warn if corresponding public key isn't registered for the domain.
 			if err := checkTokenDomain(ctx, config.Domain, config.RateLimitSigner.Public()); err != nil {
@@ -178,7 +210,8 @@ func main() {
 				log.Fatal("Writing proof failed: %v", err)
 			}
 		}
-	} else {
+	} else { // TODO: better to return above so that the "else" here is not needed?
+		// No policy specified. In this case the output should be an add-leaf request.
 		sink := func(_ string, _ *requests.Leaf) {}
 		if settings.leafHash {
 			sink = func(inputFile string, req *requests.Leaf) {
@@ -203,6 +236,16 @@ func main() {
 	}
 }
 
+func countTrue(b ...bool) int {
+	n := 0
+	for _, v := range b {
+		if v {
+			n++
+		}
+	}
+	return n
+}
+
 func (s *Settings) parse(args []string) {
 	const usage = `
 Sign checksums and submit them for logging with add-leaf requests.
@@ -258,7 +301,8 @@ If a ".req" file already exists, then it is simply overwritten.
 	versionFlag := false
 	set.FlagLong(&s.rawHash, "raw-hash", 0, "Input has already been hashed and formatted as 32 octets or a hex string")
 	set.FlagLong(&s.keyFile, "signing-key", 'k', "Private key in OpenSSH format to sign checksums; or a corresponding public key where the private part is accessed using the SSH agent protocol", "key-file")
-	set.FlagLong(&s.policyFile, "policy", 'p', "Trust policy defining logs, witnesses, and a quorum rule; omit to only output requests and exit", "policy-file")
+	set.FlagLong(&s.policyFile, "policy", 'p', "Trust policy file defining logs, witnesses, and a quorum rule; omit policy to only output requests and exit", "policy-file")
+	set.FlagLong(&s.policyName, "named-policy", 'P', "Use a named trust policy defining logs, witnesses, and a quorum rule; omit policy to only output requests and exit", "policy-name")
 	set.FlagLong(&s.leafHash, "leaf-hash", 0, "Output the request's leaf hash without submission and exit")
 	set.FlagLong(&s.outputFile, "output", 'o', "Store output in a file, only works for a single input", "output-file")
 	set.FlagLong(&s.outputDir, "output-dir", 'O', "Store output in a directory [same as corresponding input file]", "output-dir")
@@ -289,8 +333,8 @@ If a ".req" file already exists, then it is simply overwritten.
 	if len(s.outputFile) > 0 && len(s.outputDir) > 0 {
 		log.Fatal("The -o and the --output-dir options are mutually exclusive.")
 	}
-	if len(s.policyFile) > 0 && s.leafHash {
-		log.Fatal("The -p (--policy) and --leaf-hash options are mutually exclusive.")
+	if countTrue(len(s.policyName) > 0, len(s.policyFile) > 0, s.leafHash) > 1 {
+		log.Fatal("The -P, -p, and --leaf-hash options are mutually exclusive.")
 	}
 	for _, f := range s.inputFiles {
 		if len(f) == 0 {
